@@ -1,5 +1,5 @@
-/* $Id: filan.h,v 1.5 2003/06/21 17:08:13 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001, 2003 */
+/* $Id: filan.h,v 1.6 2004/11/18 20:29:10 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2004 */
 
 /* Published under the GNU General Public License V.2, see file COPYING */
 
@@ -11,7 +11,19 @@ struct sockaddr_un;	/* prevent gcc from spitting silly warning */
 struct sockaddr_in;	/* prevent gcc from spitting silly warning */
 struct sockaddr_in6;	/* prevent gcc from spitting silly warning */
 
-extern int filan(int fd, FILE *outfile);
+extern bool filan_followsymlinks;
+extern bool filan_rawoutput;
+
+extern int filan_file(const char *filename, FILE *outfile);
+extern int filan_fd(int fd, FILE *outfile);
+extern int filan_stat(
+#if HAVE_STAT64
+		      struct stat64 *buf
+#else
+		      struct stat *buf
+#endif /* !HAVE_STAT64 */
+		      , int statfd, int dynfd, FILE *outfile);
+
 extern int cdevan(int fd, FILE *outfile);
 
 #if WITH_SOCKET
