@@ -1,4 +1,4 @@
-/* $Id: xio-openssl.c,v 1.17 2005/03/18 20:23:45 gerhard Exp $ */
+/* $Id: xio-openssl.c,v 1.19 2005/09/04 09:40:45 gerhard Exp $ */
 /* Copyright Gerhard Rieger 2002-2005 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
@@ -158,7 +158,7 @@ static int
    }
 
    applyopts(-1, opts, PH_INIT);
-   applyopts_single(xfd, opts, PH_INIT);
+   if (applyopts_single(xfd, opts, PH_INIT) < 0)  return -1;
 
    retropt_bool(opts, OPT_FORK, &dofork);
 
@@ -256,6 +256,14 @@ static int
 	 }
 	 if (pid == 0) {	/* child process */
 	    Info1("just born: OpenSSL client process "F_pid, Getpid());
+
+	    xiodroplocks();	/* drop locks in xio addresses */
+	    /* some locks belong to parent, so "drop" them now */
+	    if (xiohook_newchild) {
+	       if ((*xiohook_newchild)(pid) != 0) {
+		  Exit(1);
+	       }
+	    }
 	    xfd->forever = false;
 	    xfd->retry = 0;
 	    break;
@@ -371,7 +379,7 @@ static int
    }
 
    applyopts(-1, opts, PH_INIT);
-   applyopts_single(xfd, opts, PH_INIT);
+   if (applyopts_single(xfd, opts, PH_INIT) < 0)  return -1;
 
    retropt_string(opts, OPT_OPENSSL_CERTIFICATE, &opt_cert);
    if (opt_cert == NULL) {
@@ -970,9 +978,11 @@ static int xioSSL_connect(struct single *xfd, bool opt_ver, int level) {
    return STAT_OK;
 }
 
+/* on result < 0: errno is set (at least to EIO) */
 ssize_t xioread_openssl(struct single *pipe, void *buff, size_t bufsiz) {
    unsigned long err;
    char error_string[120];
+   int _errno = EIO;	/* if we have no better idea about nature of error */
    int errint, ret;
 
    ret = sycSSL_read(pipe->para.openssl.ssl, buff, bufsiz);
@@ -996,6 +1006,7 @@ ssize_t xioread_openssl(struct single *pipe, void *buff, size_t bufsiz) {
 	    if (ret == 0) {
 	       Error("SSL_read(): socket closed by peer");
 	    } else if (ret == -1) {
+	       _errno = errno;
 	       Error1("SSL_read(): %s", strerror(errno));
 	    }
 	 } else {
@@ -1015,14 +1026,17 @@ ssize_t xioread_openssl(struct single *pipe, void *buff, size_t bufsiz) {
 	 Error("unknown error");
 	 break;
       }
+      errno = _errno;
       return -1;
    }
    return ret;
 }
 
+/* on result < 0: errno is set (at least to EIO) */
 ssize_t xiowrite_openssl(struct single *pipe, const void *buff, size_t bufsiz) {
    unsigned long err;
    char error_string[120];
+   int _errno = EIO;	/* if we have no better idea about nature of error */
    int errint, ret;
 
    ret = sycSSL_write(pipe->para.openssl.ssl, buff, bufsiz);
@@ -1046,6 +1060,7 @@ ssize_t xiowrite_openssl(struct single *pipe, const void *buff, size_t bufsiz) {
 	    if (ret == 0) {
 	       Error("SSL_write(): socket closed by peer");
 	    } else if (ret == -1) {
+	       _errno = errno;
 	       Error1("SSL_write(): %s", strerror(errno));
 	    }
 	 } else {
@@ -1065,6 +1080,7 @@ ssize_t xiowrite_openssl(struct single *pipe, const void *buff, size_t bufsiz) {
 	 Error("unknown error");
 	 break;
       }
+      errno = _errno;
       return -1;
    }
    return ret;
