@@ -1,4 +1,4 @@
-/* $Id: xio-progcall.c,v 1.45 2005/03/18 20:24:20 gerhard Exp $ */
+/* $Id: xio-progcall.c,v 1.46 2005/08/18 19:51:25 gerhard Exp $ */
 /* Copyright Gerhard Rieger 2001-2005 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
@@ -73,6 +73,7 @@ int _xioopen_foxec(char *comma,
 
    popts = moveopts(*copts, GROUP_ALL);
    applyopts2(-1, popts, PH_INIT, PH_EARLY);
+   if (applyopts_single(fd, popts, PH_INIT) < 0)  return -1;
 
    retropt_bool(popts, OPT_NOFORK, &nofork);
    withfork = !nofork;
@@ -283,7 +284,7 @@ int _xioopen_foxec(char *comma,
       /* this for parent, was after fork */
       applyopts(ptyfd, popts, PH_FD);
       applyopts(ptyfd, popts, PH_LATE);
-      applyopts_single(fd, popts, PH_LATE);
+      if (applyopts_single(fd, popts, PH_LATE) < 0)  return -1;
 
       fd->fd = ptyfd;
 
@@ -345,7 +346,7 @@ int _xioopen_foxec(char *comma,
       }
       applyopts(fd->fd, popts, PH_FD);
       applyopts(fd->fd, popts, PH_LATE);
-      applyopts_single(fd, popts, PH_LATE);
+      if (applyopts_single(fd, popts, PH_LATE) < 0)  return -1;
    } else {
       d = AF_UNIX;  type = SOCK_STREAM;
       protocol = 0;	/* PF_UNIX does not work on AIX */
@@ -382,7 +383,7 @@ int _xioopen_foxec(char *comma,
       fd->fd = sv[0];
       applyopts(fd->fd, popts, PH_FD);
       applyopts(fd->fd, popts, PH_LATE);
-      applyopts_single(fd, popts, PH_LATE);
+      if (applyopts_single(fd, popts, PH_LATE) < 0)  return -1;
    }
    /*0   if ((optpr = copyopts(*copts, GROUP_PROCESS)) == NULL)
      return STAT_RETRYLATER;*/
@@ -397,6 +398,15 @@ int _xioopen_foxec(char *comma,
 	 Error1("fork(): %s", strerror(errno));
 	 return STAT_RETRYLATER;
       }
+      if (pid == 0) {	/* child */
+	 xiodroplocks();	/* drop locks in xio addresses */
+	 /* some locks belong to parent, so "drop" them now */
+	 if (xiohook_newchild) {
+	    if ((*xiohook_newchild)(pid) != 0) {
+	       Exit(1);
+	    }
+	 }
+      }
    }
    if (!withfork || pid == 0) {	/* child */
       uid_t user;
@@ -568,9 +578,7 @@ int _xioopen_foxec(char *comma,
 #endif
    fd->para.exec.pid = pid;
 
-   if ((result = applyopts_single(fd, popts, PH_LATE)) < 0) {
-      return result;
-   }
+   if (applyopts_single(fd, popts, PH_LATE) < 0)  return -1;
    applyopts_signal(fd, popts);
    if ((numleft = leftopts(popts)) > 0) {
       Error1("%d option(s) could not be used", numleft);
