@@ -1,4 +1,4 @@
-/* $Id: xioinitialize.c,v 1.12 2005/03/13 12:19:33 gerhard Exp $ */
+/* $Id: xioinitialize.c,v 1.13 2005/08/18 19:55:36 gerhard Exp $ */
 /* Copyright Gerhard Rieger 2001-2005 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
@@ -7,10 +7,14 @@
 #include "xiosysincludes.h"
 
 #include "xioopen.h"
+#include "xiolockfile.h"
 
 
 static int xioinitialized;
 xiofile_t *sock[XIO_MAXSOCK];
+int (*xiohook_newchild)(pid_t);	/* xio calls this function from a new child
+				   process */
+
 
 int xioinitialize(void) {
    if (xioinitialized)  return 0;
@@ -75,3 +79,19 @@ int xioinitialize(void) {
    xioinitialized = 1;
    return 0;
 }
+
+
+/* well, this function is not for initialization, but I could not find a better
+   place for it
+   it is called in the child process after fork
+   it drops the locks of the xiofile's so only the parent owns them
+ */
+void xiodroplocks(void) {
+   int i;
+
+   for (i = 0; i < XIO_MAXSOCK; ++i) {
+      if (sock[i] != NULL && sock[i]->tag != XIO_TAG_INVALID) {
+	 xiofiledroplock(sock[i]);
+      }
+   }
+}
